@@ -1,3 +1,4 @@
+var posix = require("./posix");
 /*jslint onevar: true, undef: true, eqeqeq: true, plusplus: true, regexp: true, newcap: true, immed: true */
 /*globals exports, node, __filename */
 
@@ -5,33 +6,35 @@ exports.debugLevel = 0; // Increase to get more verbose debug output
 
 function debugMessage (msg) {
   if (exports.debugLevel > 0) {
-    node.error(__filename + ": " + msg.toString());
+    process.error(__filename + ": " + msg.toString());
   }
 }
 
 function debugObject (obj) {
   if (exports.debugLevel > 0) {
-    node.error(__filename + ": " + JSON.stringify(obj));
+    process.error(__filename + ": " + JSON.stringify(obj));
   }
 }
 
-exports.read = node.fs.cat;
+exports.read = posix.cat;
 
 exports.write = function (filename, data, encoding) {
-  var promise = new node.Promise();
+  var promise = new process.Promise();
 
   encoding = encoding || "utf8"; // default to utf8
 
-  node.fs.open(filename, node.O_WRONLY | node.O_TRUNC | node.O_CREAT, 0666)
+  posix.open(filename, process.O_WRONLY | process.O_TRUNC | process.O_CREAT, 0666)
     .addCallback(function (fd) {
       function doWrite (_data) {
-        node.fs.write(fd, _data, 0, encoding)
+        posix.write(fd, _data, 0, encoding)
           .addErrback(function () {
-            node.fs.close(fd);
+            posix.close(fd);
+            promise.emitError();
           })
           .addCallback(function (written) {
             if (written === _data.length) {
-              node.fs.close(fd);
+              posix.close(fd);
+              promise.emitSuccess();
             } else {
               doWrite(_data.slice(written));
             }
@@ -59,27 +62,27 @@ exports.File = function (filename, mode, options) {
 
   switch (mode) {
     case "r":
-      self.flags = node.O_RDONLY;
+      self.flags = process.O_RDONLY;
       break;
 
     case "r+":
-      self.flags = node.O_RDWR;
+      self.flags = process.O_RDWR;
       break;
 
     case "w":
-      self.flags = node.O_CREAT | node.O_TRUNC | node.O_WRONLY;
+      self.flags = process.O_CREAT | process.O_TRUNC | process.O_WRONLY;
       break;
 
     case "w+":
-      self.flags = node.O_CREAT | node.O_TRUNC | node.O_RDWR;
+      self.flags = process.O_CREAT | process.O_TRUNC | process.O_RDWR;
       break;
 
     case "a":
-      self.flags = node.O_APPEND | node.O_CREAT | node.O_WRONLY; 
+      self.flags = process.O_APPEND | process.O_CREAT | process.O_WRONLY; 
       break;
 
     case "a+":
-      self.flags = node.O_APPEND | node.O_CREAT | node.O_RDWR; 
+      self.flags = process.O_APPEND | process.O_CREAT | process.O_RDWR; 
       break;
 
     default:
@@ -108,29 +111,28 @@ proto._maybeDispatch = function () {
   debugObject(self.currentAction);
 
   args = self.currentAction.args || [];
+  method = self.currentAction.method;
 
-  if (self.currentAction.method !== "open") {
+
+  if (method !== "open") {
     args.unshift(self.fd);
   }
 
-  method = self.currentAction.method;
-
   if (!args[3] && (method === "read" || method === "write")) {
     args[3] = self.encoding;
   }
-
-  promise = node.fs[method].apply(self, args);
+  promise = posix[method].apply(self, args);
 
   userPromise = self.currentAction.promise;
 
   promise.addCallback(function () {
-    node.assert(self.currentAction.promise === userPromise);
+    process.assert(self.currentAction.promise === userPromise);
     userPromise.emitSuccess.apply(userPromise, arguments);
     self.currentAction = null;
     self._maybeDispatch();
   }).addErrback(function () {
     debugMessage("Error in method " + method);
-    node.assert(self.currentAction.promise === userPromise);
+    process.assert(self.currentAction.promise === userPromise);
     userPromise.emitError.apply(userPromise, arguments);
     self.currentAction = null;
     self._maybeDispatch();
@@ -138,27 +140,16 @@ proto._maybeDispatch = function () {
 };
 
 proto._queueAction = function (method, args) {
-  var userPromise = new node.Promise();
+  var userPromise = new process.Promise();
   this.actionQueue.push({ method: method, args: args, promise: userPromise });
   this._maybeDispatch();
   return userPromise;
 };
 
-// FIXME the following can probably be DRY'd up with some fancy getter
-// stuff.
 
-proto.open = function (filename, flags, mode) {
-  return this._queueAction("open", [filename, flags, mode]);
-};
+(["open", "write", "read", "close"]).forEach(function (name) {
+  proto[name] = function () {
+    return this._queueAction(name, Array.prototype.slice.call(arguments, 0));
+  };
+});
 
-proto.write = function (data, pos, encoding) {
-  return this._queueAction("write", [data, pos, encoding]);
-};
-
-proto.read = function (length, pos, encoding) {
-  return this._queueAction("read", [length, pos, encoding]);
-};
-
-proto.close = function () {
-  return this._queueAction("close");
-};
