@@ -4,7 +4,7 @@ var promise = new process.Promise();
 var cancelled = false;
 promise.addCancelback(function(){
   if(cancelled){
-    assertUnreachable("promise should not cancel more than once");
+    assert.ok(false, "promise should not cancel more than once");
   }
   cancelled = true;
 });
