@@ -8,9 +8,10 @@ var f2 = path.join(fixturesDir, "x2.txt");
 puts("watching for changes of " + f);
 
 var changes = 0;
-process.watchFile(f, function () {
+process.watchFile(f, function (curr, prev) {
   puts(f + " change");
   changes++;
+  assert.equal(true, curr.mtime != prev.mtime);
   process.unwatchFile(f);
 });
 
@@ -23,5 +24,5 @@ file.close().wait();
 
 
 process.addListener("exit", function () {
-  assertTrue(changes > 0);
+  assert.equal(true, changes > 0);
 });
